@@ -45,12 +45,13 @@ A tiny example:
 __docformat__ = "restructuredtext en"
 
 # The format of the __version__ line is matched by a regex in setup.py
-__version__ = "6.1.1"
-__date__ = "2021-12-20"
+__version__ = "6.2.0"
+__date__ = "2022-12-20"
 
 __all__ = [
     "URIRef",
     "BNode",
+    "IdentifiedNode",
     "Literal",
     "Variable",
     "Namespace",
@@ -83,10 +84,12 @@ __all__ = [
     "VOID",
     "XSD",
     "util",
+    "plugin",
+    "query",
 ]
 
-import sys
 import logging
+import sys
 
 logger = logging.getLogger(__name__)
 
@@ -96,6 +99,7 @@ try:
     if (
         not hasattr(__main__, "__file__")
         and sys.stdout is not None
+        and hasattr(sys.stderr, "isatty")
         and sys.stderr.isatty()
     ):
         # show log messages in interactive mode
@@ -155,13 +159,8 @@ In particular, this determines how the rich comparison operators for
 Literal work, eq, __neq__, __lt__, etc.
 """
 
-from rdflib.term import URIRef, BNode, Literal, Variable
-
-from rdflib.graph import Dataset, Graph, ConjunctiveGraph
-
-from rdflib import plugin
-from rdflib import query
 
+from rdflib.graph import ConjunctiveGraph, Dataset, Graph
 from rdflib.namespace import (
     BRICK,
     CSVW,
@@ -191,11 +190,7 @@ from rdflib.namespace import (
     XSD,
     Namespace,
 )
+from rdflib.term import BNode, IdentifiedNode, Literal, URIRef, Variable
 
-# tedious sop to flake8
-assert plugin
-assert query
-
-from rdflib import util
-
-from rdflib.container import *
+from rdflib import plugin, query, util  # isort:skip
+from rdflib.container import *  # isort:skip # noqa:F401,F403
