@@ -1,13 +1,7 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 from rdflib.namespace import RDF
-from rdflib.term import BNode
-from rdflib.term import Literal
+from rdflib.term import BNode, Literal
 
-
-__all__ = ['Collection']
+__all__ = ["Collection"]
 
 
 class Collection(object):
@@ -18,15 +12,21 @@ class Collection(object):
     >>> from rdflib.graph import Graph
     >>> from pprint import pprint
     >>> listName = BNode()
-    >>> g = Graph('IOMemory')
+    >>> g = Graph('Memory')
     >>> listItem1 = BNode()
     >>> listItem2 = BNode()
-    >>> g.add((listName, RDF.first, Literal(1)))
-    >>> g.add((listName, RDF.rest, listItem1))
-    >>> g.add((listItem1, RDF.first, Literal(2)))
-    >>> g.add((listItem1, RDF.rest, listItem2))
-    >>> g.add((listItem2, RDF.rest, RDF.nil))
-    >>> g.add((listItem2, RDF.first, Literal(3)))
+    >>> g.add((listName, RDF.first, Literal(1))) # doctest: +ELLIPSIS
+    <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+    >>> g.add((listName, RDF.rest, listItem1)) # doctest: +ELLIPSIS
+    <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+    >>> g.add((listItem1, RDF.first, Literal(2))) # doctest: +ELLIPSIS
+    <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+    >>> g.add((listItem1, RDF.rest, listItem2)) # doctest: +ELLIPSIS
+    <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+    >>> g.add((listItem2, RDF.rest, RDF.nil)) # doctest: +ELLIPSIS
+    <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+    >>> g.add((listItem2, RDF.first, Literal(3))) # doctest: +ELLIPSIS
+    <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
     >>> c = Collection(g,listName)
     >>> pprint([term.n3() for term in c])
     [u'"1"^^<http://www.w3.org/2001/XMLSchema#integer>',
@@ -52,22 +52,28 @@ class Collection(object):
         """
         >>> from rdflib.graph import Graph
         >>> listName = BNode()
-        >>> g = Graph('IOMemory')
+        >>> g = Graph('Memory')
         >>> listItem1 = BNode()
         >>> listItem2 = BNode()
-        >>> g.add((listName, RDF.first, Literal(1)))
-        >>> g.add((listName, RDF.rest, listItem1))
-        >>> g.add((listItem1, RDF.first, Literal(2)))
-        >>> g.add((listItem1, RDF.rest, listItem2))
-        >>> g.add((listItem2, RDF.rest, RDF.nil))
-        >>> g.add((listItem2, RDF.first, Literal(3)))
+        >>> g.add((listName, RDF.first, Literal(1))) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((listName, RDF.rest, listItem1)) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((listItem1, RDF.first, Literal(2))) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((listItem1, RDF.rest, listItem2)) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((listItem2, RDF.rest, RDF.nil)) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((listItem2, RDF.first, Literal(3))) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
         >>> c = Collection(g, listName)
         >>> print(c.n3()) #doctest: +NORMALIZE_WHITESPACE
         ( "1"^^<http://www.w3.org/2001/XMLSchema#integer>
           "2"^^<http://www.w3.org/2001/XMLSchema#integer>
           "3"^^<http://www.w3.org/2001/XMLSchema#integer> )
         """
-        return "( %s )" % (' '.join([i.n3() for i in self]))
+        return "( %s )" % (" ".join([i.n3() for i in self]))
 
     def _get_container(self, index):
         """Gets the first, rest holding node at index."""
@@ -103,8 +109,7 @@ class Collection(object):
                 elif not newLink:
                     raise Exception("Malformed RDF Collection: %s" % self.uri)
                 else:
-                    assert len(newLink) == 1, \
-                        "Malformed RDF Collection: %s" % self.uri
+                    assert len(newLink) == 1, "Malformed RDF Collection: %s" % self.uri
                     listName = newLink[0]
 
     def __getitem__(self, key):
@@ -136,12 +141,18 @@ class Collection(object):
         >>> a = BNode('foo')
         >>> b = BNode('bar')
         >>> c = BNode('baz')
-        >>> g.add((a, RDF.first, RDF.type))
-        >>> g.add((a, RDF.rest, b))
-        >>> g.add((b, RDF.first, RDFS.label))
-        >>> g.add((b, RDF.rest, c))
-        >>> g.add((c, RDF.first, RDFS.comment))
-        >>> g.add((c, RDF.rest, RDF.nil))
+        >>> g.add((a, RDF.first, RDF.type)) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((a, RDF.rest, b)) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((b, RDF.first, RDFS.label)) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((b, RDF.rest, c)) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((c, RDF.first, RDFS.comment)) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
+        >>> g.add((c, RDF.rest, RDF.nil)) # doctest: +ELLIPSIS
+        <Graph identifier=... (<class 'rdflib.graph.Graph'>)>
         >>> len(g)
         6
         >>> def listAncestry(node, graph):
@@ -218,6 +229,7 @@ class Collection(object):
 
         self.graph.add((end, RDF.first, item))
         self.graph.add((end, RDF.rest, RDF.nil))
+        return self
 
     def __iadd__(self, other):
 
@@ -233,6 +245,7 @@ class Collection(object):
             self.graph.add((end, RDF.first, item))
 
         self.graph.add((end, RDF.rest, RDF.nil))
+        return self
 
     def clear(self):
         container = self.uri
@@ -242,48 +255,4 @@ class Collection(object):
             graph.remove((container, RDF.first, None))
             graph.remove((container, RDF.rest, None))
             container = rest
-
-
-def test():
-    import doctest
-    doctest.testmod()
-
-
-if __name__ == "__main__":
-    test()
-
-    from rdflib import Graph
-    g = Graph()
-
-    c = Collection(g, BNode())
-
-    assert len(c) == 0
-
-    c = Collection(
-        g, BNode(), [Literal("1"), Literal("2"), Literal("3"), Literal("4")])
-
-    assert len(c) == 4
-
-    assert c[1] == Literal("2"), c[1]
-
-    del c[1]
-
-    assert list(c) == [Literal("1"), Literal("3"), Literal("4")], list(c)
-
-    try:
-        del c[500]
-    except IndexError as i:
-        pass
-
-    c.append(Literal("5"))
-
-    print(list(c))
-
-    for i in c:
-        print(i)
-
-    del c[3]
-
-    c.clear()
-
-    assert len(c) == 0
+        return self
